/*!
\file skytexture.cpp
\brief Class definition for SkyTexture
\author Nicholas Phillips, phillips@friday.gsfc.nasa.gov
\date Feb 23, 2004
*/
#include <iostream>
#include <iomanip>
#include "chealpix.h"
#include "skytexture.h"

using namespace std;
//================================================
long xy2pix(long ix, long iy) {
	static int x2pix[128];
	static int y2pix[128];
	static bool xy2pix_called = false;
	if( ! xy2pix_called ) {
		mk_xy2pix(x2pix, y2pix);
	xy2pix_called = true;
	}
	//int ix_low = (int)fmod(ix,128);
	int ix_low = ix % 128;
	int ix_hi  = ix / 128;
	int iy_low = iy % 128;
	int iy_hi  = iy / 128;
	return (x2pix[ix_hi]+y2pix[iy_hi]) * (128 * 128)
			+ (x2pix[ix_low]+y2pix[iy_low]);
}
/*
//==============================================
void pix2fxy(long nside, long pix, long &face, long &x, long &y) {
	face = pix/(nside*nside);
	long facepix = pix % (nside*nside);
	x = facepix / nside;
	y = facepix % nside;
	return;
}
*/
//======================================================================================
SkyTexture::SkyTexture() : texture(0), texture_res(0), nside(0)
{
	hilite_level = 128;
	select_level = 64;
}


//======================================================================================
SkyTexture::~SkyTexture()
{
	if( texture ) delete[] texture;
}


//======================================================================================
PixLUTCache::iterator SkyTexture::getLUT(const int nside_, const bool ordering)
{
	//cout << "SkyTexture::getLUT(" <<  nside_ << ", " <<  ordering << ")" << endl;

	PixLUTCache &lut_cache = ordering ? lut_cache_ring : lut_cache_nest;

	PixLUTCache::iterator luti = lut_cache.find(nside_);
	if( luti == lut_cache.end() )
			buildLUT(nside_, ordering);
	luti = lut_cache.find(nside_);
	if( luti == lut_cache.end() ) {
		cerr << "SkyTexture::getLUT:Failed, aborting." << endl;
		abort();
	}
	return luti;
}
//======================================================================================
bool SkyTexture::buildLUT(const int ns, const bool ordering)
{
	//cout << "SkyTexture::buildLUT(" <<  ns << ", " <<  ordering << ")" << endl;

	PixLUTCache &lut_cache = ordering ? lut_cache_ring : lut_cache_nest;
	PixLUT &lut = lut_cache[ns];
	lut.resize(12*ns*ns);

	long dy = 4*ns;
	long pix;
	long k = 0;
	for(int face = 0; face < 12; face++) {
		long xo = ns*(face % 4);
		long yo = ns*(face / 4);
		long face_offset = face*ns*ns;
		for(int y = 0; y < ns; y++) {
			for(int x = 0; x < ns; x++) {
				k = x + xo + (y+yo)*dy;
				pix = xy2pix(x,y)  + face_offset;
				if( ordering )
					nest2ring(ns,pix,&pix);
				lut[pix] = 4*k;
			}
		}
	}
	return true;
}
//======================================================================================
void SkyTexture::set(const int nside_, const bool ordering, const ColorTable &skycolor)
{
    //cout << "SkyTexture::set()" << endl;
	lut = &(getLUT(nside_, ordering)->second);
	// Didn't abort, assume all is good
	if( nside_ != nside ) {
		nside = nside_;
		texture_res = 4*nside;
		if( texture) delete[] texture;
		/*
		skytexture.cpp: In member function ‘void SkyTexture::set(int, bool, const ColorTable&)’:
		skytexture.cpp:110: error: array bound forbidden after parenthesized type-id
		skytexture.cpp:110: note: try removing the parentheses around the type-id
		*/
		//texture = new (unsigned char)[texture_res*texture_res*4];
		texture = new unsigned char[texture_res*texture_res*4];
	}

	long texk = 0;
	uint color;
	long npix = 12*nside*nside;
	for(long pix = 0; pix < npix; pix++) {
		color = skycolor[pix];
		texk = (*lut)[pix];
		texture[texk++] = qRed(color);
		texture[texk++] = qGreen(color);
		texture[texk++] = qBlue(color);
		texture[texk++] = 255;
	}
	return;
}


//======================================================================================
void SkyTexture::glTexture()
{
	glTexImage2D(
		GL_TEXTURE_2D, 0, 4,
		texture_res, texture_res, 0,
		GL_RGBA, GL_UNSIGNED_BYTE,
		texture
	);
    return;
}


//======================================================================================
void SkyTexture::highlight(const int pixnum, const bool on)
{
	if( pixnum < 0 )
		return;

	long texk = (*lut)[pixnum];
	texture[texk+3] = on ? hilite_level : 255;
	return;
}


//======================================================================================
void SkyTexture::select(const int pixnum, const bool on)
{
	if( pixnum < 0 )
		return;

	long texk = (*lut)[pixnum];
	texture[texk+3] = on ? select_level : 255;
	return;
}
